/**
 * This script is public domain stuff. 
 * You use it on your own risk. 
 * We accept no liability on its behavior.
 * 
 * This script reads data from a Adafruit Color Sensor (TCS34725) and sends it to a server via WiFi.
 */
#include <Wire.h>
#include "Adafruit_TCS34725.h"
#include <ESP8266WiFi.h>

const char* ssid     = "ZUMMOBOT-NET5";
const char* password = "passwordgoeshere";
const char* host = "192.168.0.3";
int valueCount = 0;

/* Based on example code for the Adafruit TCS34725 breakout library */

/* Connect SCL    to SCL
   Connect SDA    to SDA
   Connect VDD    to 3.3V DC
   Connect GROUND to common ground */
   
/* Initialise with default values (int time = 2.4ms, gain = 1x) */
// Adafruit_TCS34725 tcs = Adafruit_TCS34725();

/* Initialise with specific int time and gain values */
Adafruit_TCS34725 tcs = Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_2_4MS, TCS34725_GAIN_1X);

void setup(void) {
  Serial.begin(115200);
  initSensor();
  initWiFi();
}

void loop(void) {
  uint16_t r, g, b, c;
  
  tcs.getRawData(&r, &g, &b, &c);
 
  String dataToSend = craftDataToSend(r,g,b,c);
  sendValueToWiFi(dataToSend);

}

String craftDataToSend(uint16_t r, uint16_t g, uint16_t b, uint16_t c){

  Serial.print("R: "); Serial.print(r, DEC); Serial.print(" ");
  Serial.print("G: "); Serial.print(g, DEC); Serial.print(" ");
  Serial.print("B: "); Serial.print(b, DEC); Serial.print(" ");
  Serial.print("C: "); Serial.print(c, DEC); Serial.print(" ");
  Serial.println(" ");  
  String dataToSend = "&R=";
  dataToSend+=String(r,DEC);
  dataToSend+="&G=";
  dataToSend+=String(g,DEC);
  dataToSend+="&B=";
  dataToSend+=String(b,DEC);
  dataToSend+="&C=";
  dataToSend+=String(c,DEC);

  return dataToSend;  
}

void sendValueToWiFi(String data){
  valueCount ++;
  Serial.print("connecting to ");
  Serial.println(host);
  
  // Use WiFiClient class to create TCP connections
  WiFiClient client;
  const int httpPort = 80;
  if (!client.connect(host, httpPort)) {
    Serial.println("connection failed");
    return;
  }
  
  // We now create a URI for the request
  String url = "/zummobotdb/set.php";
  url += "?MS=";
  url += String(millis());
  url += "&VC=";
  url += String(valueCount);
  url += data;

  Serial.print("Requesting URL: ");
  Serial.println(url);
  
  // This will send the request to the server
  String request = String("GET ") + url + " HTTP/1.1\r\n" +
               "Host: " + host + "\r\n" + 
               "Connection: close\r\n\r\n";

  Serial.println("REQUEST:");
  Serial.println(request);
               
  client.print(request);
  client.stop();
  
}

void initWiFi(){
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);  

   /* Explicitly set the ESP8266 to be a WiFi-client, otherwise, it by default,
     would try to act as both a client and an access-point and could cause
     network-issues with your other WiFi-devices on your WiFi-network. */
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected");  
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());  
}

bool initSensor(){
  if (tcs.begin()) {
    Serial.println("Found sensor");
  } else {
    Serial.println("No TCS34725 found ... check your connections");
    while (true);
  }
}
